﻿using SABClientNSwag;
using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using Newtonsoft.Json;

namespace TDP.Utils
{
  public static class ExceptionUtils
  {
    internal static string FormatException(Exception e,Func<string,string> formatMessage)
    {
      if (e is AggregateException ae)
      {
        return FormatExceptionMessages(ae.Flatten().InnerExceptions);
      }
      else if (e is ApiException<ErrorResponse> ar)
      {
        return formatMessage(GetErrorMessage(ar.Result,ar.Message));
      }
      else if (e is ApiException apiE)
      {
        if (TryParseErrorResponse(apiE.Response, out ErrorResponse errorResponse))
        {
          return formatMessage(GetErrorMessage(errorResponse,apiE.Message));
        }

        return FormatExceptionMessages(apiE);
      }
      else
      {
        return FormatExceptionMessages(e);
      }
    }

    internal static string GetErrorMessage(ErrorResponse resp, string fallbackMessage)
    {
      var ret = fallbackMessage;

      if (resp == null || resp.Msg==null || resp.Msg.Count==0) return ret;

      return resp.Msg.ElementAt(0).Message;
    }


    internal static string FormatExceptionMessages(Exception e)
    {
      var sb = new StringBuilder(500);

      AppendInnerException(sb, e, "");
      sb.Append("\r\n");

      return sb.ToString();
    }

    internal static string FormatExceptionMessages(IList<Exception> exceptions)
    {
      var sb = new StringBuilder(500);

      foreach (Exception e in exceptions)
      {
        AppendInnerException(sb, e, "");
        sb.Append("\r\n");
      }

      return sb.ToString();
    }

    internal static void AppendInnerException(StringBuilder sb, Exception e, string prefix)
    {
      sb.Append(prefix);
      sb.Append(e.Message);
      sb.Append("\r\n");
      if (e.InnerException != null) AppendInnerException(sb, e.InnerException, "");
    }

    internal static bool TryParseErrorResponse(string response, out ErrorResponse errorResponse)
    {
      try
      {
        var err = (ErrorResponse)Newtonsoft.Json.JsonConvert.DeserializeObject(response, typeof(ErrorResponse));

        errorResponse = err;

        if (errorResponse == null)
          return false;
      }
      catch (JsonException)
      {
        errorResponse = null;
        return false;
      }

      return true;
    }

    internal static bool WasLoggedOut(Exception e)
    {
      if(e is ApiException<ErrorResponse> ar)
      {
        return "m:login.failed".Equals(GetErrorMessage(ar.Result,"Your session is not authenticated!"));
      }

      return false;
    }

    internal static string FormatException(Exception e)
    {
      return FormatException(e, (msg) => msg);
    }
  }
}
