﻿using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Serialization;
using System;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Threading.Tasks;
using SABClientExtensions;
using TDP.Utils;
using SABClientNSwag;

namespace SABClientExample
{
  class Program
  {
    static void Main(string[] args)
    {
      // We need HttpClient object to communicate and automaticcaly manage open connection(s) to SecureAnyBox (SAB) server and keep login session cookies
      // You should dispose http client only after you finish communicating with SAB server.
      var http = new HttpClient();

      // Same with SABClientNSwag.Client ... keep one for all requests to SAB server. It just wraps http client and sends REST requests and decodes responses.
      // Client class was generated directly from swagger.json SAB API schema using NSwag tool
      var client = new Client("https://192.168.1.1/sab/", http);


      // configure JsonConverter serialization and parsing settings to properly decode enum types and CamelCase naming of properties
      SerializeJsonEnumsAsStrings();

      Run(client).Wait();

      Console.WriteLine("done");
    }

    private static void SerializeJsonEnumsAsStrings()
    {
      JsonConvert.DefaultSettings = () =>
      {
        var stringEnumConverter = new StringEnumConverter { NamingStrategy = new CamelCaseNamingStrategy() };
        var settings = new JsonSerializerSettings();
        settings.Converters.Add(stringEnumConverter);
        return settings;
      };
    }


    private static async Task Run(Client client)
    {
      // access code is transfered in http header and must be URL encoded (UTF-8) to preserve international and special characters 
      // HTTP protocol doesn't have standard for encoding header values and SecureAnyBox server accepts URL encoded access code
      string ACCESS_CODE = Uri.EscapeDataString("apitest123");

      try
      {
        const string Login = "System\\apitest";
        const string Password = "apitest";
        
        var response = await client.LoginAsync(Login, Password);

        // we could check response.SecondFactorRequired and ask user to provide 2FA code (client.Authenticate2FAsync)
        // we skip it in this example as apitest user has no second factor configured

        // we list all writeable (create + write) Safe Boxes (and Safe Box Groups) under root
        const string WriteAccess = "CW";
        SafeBoxes boxes = await client.GetSafeBoxesAsync(WriteAccess);

        foreach(var box in boxes.Safeboxes)
        {
          Console.WriteLine("Safe Box: '"+box.Name+"'");
          
          if (!box.IsSafeBox()) continue; // skip Safe Box Groups

          await TryCreateFileRecord(client, ACCESS_CODE, box); // we try to create a file recrod in each Safe Box
          await ListRecords(client, box); // and list all records in the Safe Box
        }

      } catch (Exception e)
      {        
        Console.WriteLine("error: " + ExceptionUtils.FormatException(e));
      }
    }

    private static async Task ListRecords(Client client, SafeBox box)
    {
      Records records = await client.GetRecordsAsync(box.Id ?? 0, true);

      foreach (var rec in records.Records1)
      {
        Console.WriteLine(" "+rec.Name);
      }

      Console.WriteLine(records.Records1.Count.ToString()+" record(s)");
      Console.WriteLine("");
    }

    private static async Task TryCreateFileRecord(Client client, string ACCESS_CODE, SafeBox box)
    {
      // file record is created in two steps
      // first we create the file record with name and other identification fields (CreateRecprdAsync)
      // then we upload file linked to this record (UploadAsync)
      var filerec = new FileRecord
      {
        Template = FileRecordTemplate.File,
        Name = "test file",
        Note = "created from API"
      };

      try
      {
        var newrec = await client.CreateRecordAsync(filerec, box.Id.Value, ACCESS_CODE);

        FileParameter file = new SABClientNSwag.FileParameter(new FileStream("d:\\test.jpg", FileMode.Open, FileAccess.Read), "test.jpg", "binary/octet-stream");

        // uploaded file is stored as field "file" of previously created record (newrec)
        string respStr = await readAsString(await client.UploadAsync("apitest123", newrec.Id, "file", file));
      }
      catch (ApiException<ErrorResponse> e)
      {
        var err = ExceptionUtils.GetErrorMessage(e.Result, e.Message);

        // if the file already exists, just report it
        if (SABErrors.DUPLICATE_FILE.Equals(err))
        {
          Console.WriteLine("file '" + filerec.Name + "' already exists in Safe Box '" + box.Name + "'");
        }
        else
        {
          // we fail if another error was thrown
          throw e;
        }
      }
    }

    // for complicated reasons (mostly older browser compatibility) file upload resource returns simple string response
    // NSwag currently doesn't support these and we must read the raw response stream and convert it to string
    private static async Task<string> readAsString(SABClientNSwag.FileResponse resp)
    {
      using (StreamReader reader = new StreamReader(resp.Stream))
      {
        return await reader.ReadToEndAsync();
      }      
    }
  }
}
